/*
 * File: plug-ins/xslt/xslt.c
 * 
 * Made by Matthieu Sozeau <mattam@netcourrier.com>
 * 
 * Started on  Thu May 16 23:21:43 2002 Matthieu Sozeau
 * Last update Mon Jun  3 20:07:11 2002 Matthieu Sozeau
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <config.h>
#include <stdio.h>
#include "filter.h"
#include "plug-ins.h"
#include "intl.h"
#include "message.h"
#include "dia_dirs.h"
#include <libxslt/xslt.h>
#include <libxslt/xsltInternals.h>
#include <libxslt/transform.h>
#include <libxslt/xsltutils.h>

#include "xslt.h"

void xslt_ok(void);

static char *diafilename;
static char *filename;

static GModule *xslt_module;

static char *stylesheets[] = {
	UML_TO_CPP_STYLESHEET,
	UML_TO_IDL_STYLESHEET,
	UML_TO_JAVA_STYLESHEET
};

static void
export_data(DiagramData *data, const gchar *f, 
            const gchar *diaf, void* user_data)
{
	if(filename != NULL)
		g_free(filename);
	
	filename = g_strdup(f);
	if(diafilename != NULL)
		g_free(diafilename);

	diafilename = g_strdup(diaf);

	xslt_dialog_create();	
}


void xslt_ok() {
	FILE *file, *out;
	int err;
	gchar *stylefname;
	
	xsltStylesheetPtr style, codestyle;
	xmlDocPtr doc, res;

	file = fopen(diafilename, "r");

	if (file == NULL) {
		message_error(_("Couldn't open: '%s' for reading.\n"), diafilename);
		return;
	}

	out = fopen(filename, "w+");
	if (out == NULL) {
		message_error(_("Couldn't open: '%s' for writing.\n"), filename);
		return;
	}
	
	xmlSubstituteEntitiesDefault(0);
	doc = xmlParseFile(diafilename);

	if(doc == NULL) {
		message_error(_("Error while parsing %s\n"), diafilename);
		return;
	}
	
	stylefname = g_strconcat(g_getenv("DIA_PLUGIN_PATH"),
				 G_DIR_SEPARATOR_S, "xslt", 
				 G_DIR_SEPARATOR_S, DIA_TO_UML_STYLESHEET, NULL);

	//stylefname = malloc(strlen() + strlen(DIA_TO_UML_STYLESHEET) + 1); 
	//sprintf(stylefname, "%s%s", g_getenv("DIA_PLUGIN_PATH"), DIA_TO_UML_STYLESHEET);
	
	style = xsltParseStylesheetFile((const xmlChar *) stylefname);
	if(style == NULL) {
		message_error(_("Error while parsing stylesheet %s\n"), stylefname);
		return;
	}
	
	res = xsltApplyStylesheet(style, doc, NULL);
	if(res == NULL) {
		message_error(_("Error while applying stylesheet\n"));
		return;
	}
	
	g_free(stylefname);
	
	/*stylefname = malloc(strlen(g_getenv("DIA_PLUGIN_PATH")) + strlen(stylesheets[language]) + 1); 
	sprintf(stylefname, "%s%s", g_getenv("DIA_PLUGIN_PATH"), stylesheets[language]);
	*/
	stylefname = g_strconcat(g_getenv("DIA_PLUGIN_PATH"), 
				 G_DIR_SEPARATOR_S, "xslt", 
				 G_DIR_SEPARATOR_S, stylesheets[language], NULL);

	codestyle = xsltParseStylesheetFile((const xmlChar *) stylefname);
	if(codestyle == NULL) {
		message_error(_("Error while parsing stylesheet: %s\n"), stylesheets[language]);
		return;
	}
	
	xmlFreeDoc(doc);
	
	doc = xsltApplyStylesheet(codestyle, res, NULL);
	if(doc == NULL) {
		message_error(_("Error while applying stylesheet: %s\n"), stylesheets[language]);
		return;
	}


	err = xsltSaveResultToFile(out, doc, codestyle);
	//printf("Saved to file (%i) using stylesheet: %s\n", err, stylefname);
		
	fclose(out);
	fclose(file);
	g_free(stylefname);

	xsltFreeStylesheet(codestyle);
	xsltFreeStylesheet(style);
	xmlFreeDoc(res);
	xmlFreeDoc(doc);
	xsltCleanupGlobals();
	xmlCleanupParser();

	xslt_clear();
}

/* --- dia plug-in interface --- */

#define MY_RENDERER_NAME "code"

static const gchar *extensions[] = { "code", NULL };
static DiaExportFilter my_export_filter = {
	N_(MY_RENDERER_NAME),
	extensions,
	export_data
};


PluginInitResult
dia_plugin_init(PluginInfo *info)
{
	gchar *path;
	if (!dia_plugin_info_init(info, "XSLT",
			          _("XSL Transformation filter"),
			          NULL, NULL))
	{
			return DIA_PLUGIN_INIT_ERROR;
	}
	
#ifdef G_OS_WIN32
	path = g_module_build_path("\Windows", "xslt");
#else
	path = g_module_build_path("/usr/lib", "xslt");
#endif	
	xslt_module = g_module_open(path, 0);
	if(!xslt_module) {
		message_error("Could not load XSLT library (%s) : %s", path, g_module_error());
		return DIA_PLUGIN_INIT_ERROR;
	}
	filter_register_export(&my_export_filter);
		
	g_free(path);

	return DIA_PLUGIN_INIT_OK;
}
